package com.easemob.easemob_im;

import com.hyphenate.exceptions.HyphenateException;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import androidx.annotation.NonNull;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.JSONMethodCodec;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;



public class EMWrapper implements MethodChannel.MethodCallHandler {

  private static final String CHANNEL_PREFIX = "com.easemob.im/";

  private final ExecutorService cachedThreadPool = Executors.newCachedThreadPool();

  public EMWrapper(FlutterPlugin.FlutterPluginBinding binding, String channelName) {
      this.flutterPluginBinding = binding;
      this.channel = new MethodChannel(binding.getBinaryMessenger(), CHANNEL_PREFIX + channelName, JSONMethodCodec.INSTANCE);
      channel.setMethodCallHandler(this);
  }

  public MethodChannel channel;
  public FlutterPlugin.FlutterPluginBinding flutterPluginBinding;

  public void post(Runnable runnable) {
    EasemobImPlugin.handler.post(runnable);
  }

  public void asyncRunnable(Runnable runnable) {
    cachedThreadPool.execute(runnable);
  }

  public void onSuccess(MethodChannel.Result result, String channelName, Object object) {
    post(()-> {
      Map<String, Object> data = new HashMap<>();
      if (object != null) {
        data.put(channelName, object);
      }
      result.success(data);
    });
  }

  public void onError(MethodChannel.Result result, HyphenateException e) {
    post(()-> {
      Map<String, Object> data = new HashMap<>();
        data.put("error", HyphenateExceptionHelper.toJson(e));
        result.success(data);
    });
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
    result.notImplemented();
  }
}
